from time import sleep, ctime
import urllib.request as url
from urllib.error import URLError, HTTPError
import json, time
from geopy.distance import geodesic
try:
    from dbsql.dbconnections import *
except ModuleNotFoundError:
    from liveisstracker.dbsql.dbconnections import *
from geopy.geocoders import Nominatim
from mylogger.iss_logging import logger

geolocator = Nominatim(user_agent="my-application",timeout=3)


def get_city_location(city_name):
    try:
        location = geolocator.geocode(city_name)
        if not location:
            logger.error(f'"{city_name}" is not a valid city name')
            return 'Invalid','0.0','0.0'
        logger.info(location)
        logger.debug(location.address+"Latitude: "+location.raw['lat'])
        logger.debug(location.address+"Longitude: "+location.raw['lon'])
    except Exception as e:
        logger.error(f'Encountered {e} while trying to get city location')
        return 'Invalid','0.0','0.0'

    return location.address, location.raw['lat'], location.raw['lon']

class TrackerISS:

    lat_pre = 0
    lon_pre = 0
    timestamp_pre = 0
    iss_link = 'http://api.open-notify.org/iss-now.json'
    pass_link = 'http://api.open-notify.org/iss-pass.json?lat=LAT&lon=LON'

    def __init__(self):

        gps_location = self.get_iss_lat_lon()
        self.timestamp = gps_location['timestamp']
        self.latitude = gps_location['latitude']
        self.longitude = gps_location['longitude']

        try:
            logger.info('Establishing connection to DB')
            self.db_cnx = MySql()
        except Exception:
            logger.error('DB connection failed')
            self.db_cnx = None

    def insert_record_in_db(self, lat, lon, timestamp):

        if self.db_cnx is not None:
            logger.info(f'Inserting into DB lat:{lat},lon:{lon},timestamp:{timestamp}')
            self.db_cnx.insert_record('location',key_values={'lat':lat,'lon':lon,'datetime_id':timestamp})
    
    @staticmethod
    def get_distance_btwn_locations(location_1=None, location_2=None,testing_mode=None):
        
        if testing_mode:
            location_1 = testing_mode['location_1']
            location_2 = testing_mode['location_2']

        return geodesic(location_1,location_2).km

    @staticmethod
    def get_iss_lat_lon(testing_mode=None):

        try:
            if testing_mode:
                response = url.urlopen(testing_mode['iss_link'])
            else:
                logger.info('Getting ISS stat')
                response = url.urlopen(TrackerISS.iss_link)
            json_res = json.loads(response.read())
            geo_location = json_res['iss_position']
            timestamp = json_res['timestamp']
            lon, lat = float(geo_location['longitude']), float(geo_location['latitude'])
            logger.debug(f"Current ISS location at {ctime(int(timestamp))}: latitude: {lat}, longitude: {lon}")
            return {'timestamp':timestamp, 'latitude': lat,'longitude': lon}
        except URLError as e:
            raise e
    
    @staticmethod
    def get_pass_info_from_lat_lon(lat=None,lon=None,testing_mode=None):
        if testing_mode:
            pass_url = TrackerISS.pass_link.replace('LAT', str(testing_mode['latitude'])).replace('LON', str(testing_mode['longitude']))
        else:
            pass_url = TrackerISS.pass_link.replace('LAT', str(lat)).replace('LON', str(lon))
        try:
            logger.info(f'Getting ISS pass through info for lat:{lat},lon:{lon}')
            response = url.urlopen(pass_url)
        except url.HTTPError as e:
            logger.error(f'HTTP error while opening URL "{pass_url}"; Bad request: Error code:{e.code}')
            return 'failure'
        json_res = json.loads(response.read())
        if json_res['message'] == 'success':
            return json_res['response']
        else:
            return json_res['message']

    def get_speed_iss_pos(self,ignore_db_insert=False,testing_mode=None):

        if testing_mode:
            self.timestamp = testing_mode['timestamp']
            self.latitude = testing_mode['latitude']
            self.longitude = testing_mode['longitude']
        else:

            gps_location = self.get_iss_lat_lon()
            self.timestamp = gps_location['timestamp']
            self.latitude = gps_location['latitude']
            self.longitude = gps_location['longitude']

            if not ignore_db_insert:
                # Write location stats to DB
                self.insert_record_in_db(self.latitude, self.longitude, self.timestamp)

        # global lat_pre,lon_pre, self.timestamp_pre

        iss = (self.latitude, self.longitude)
        time_diff = self.timestamp - self.timestamp_pre
        distance = geodesic((self.lat_pre,self.lon_pre),iss).km
        self.lat_pre,self.lon_pre = iss
        self.timestamp_pre = self.timestamp

        try:
            speed = distance/time_diff*3600 # km/h
        except ZeroDivisionError:
            speed = 0
        except  Exception as e:
            logger.error(e)
            speed = 99999999999

        return [speed, iss]



import plotly.graph_objects as go
from mylogger.iss_logging import logger
import streamlit as st
from issTrack.issTracking import *
from time import sleep
import os
import pytz
from dateutil import tz
from datetime import datetime

# co_ordinate=iss.get_speed_iss_pos()
# latitude=co_ordinate[1][0]
# longitude=co_ordinate[1][1]

def get_plotly_figure(location,token):
    latitude = location[1][0]
    longitude = location[1][1]
    fig = go.Figure(go.Scattermapbox(lat=[str(latitude)],lon=[str(longitude)],mode='markers',marker=go.scattermapbox.Marker(size=14)))
    fig.update_layout(hovermode='closest',mapbox=dict(accesstoken=token,bearing=0, center=go.layout.mapbox.Center(lat=location[1][0],lon=location[1][1]),pitch=0,zoom=0))
    return fig

def get_mapbox_token(env_var_name):

    try:
        mapbox_access_token = open(os.getenv(env_var_name)).read()
        logger.debug(f'MAPBOX Access Token: {mapbox_access_token}')
        return False, mapbox_access_token
    except TypeError:
        logger.error('Not a valid token. MAPBOX chart will be disabled')
        return True, None

def display_pass_statistics(pass_information):
        logger.debug(f'Pass information: {pass_information}')
        if not pass_information == 'failure':
            pass_number = st.selectbox('Select the pass number?',tuple([s+1 for s in range(len(pass_information))]))
            visible_duration = pass_information[pass_number-1]['duration']
            pred_time = pass_information[pass_number-1]['risetime'] # .strftime('%Y-%m-%d %H:%M:%S') GMT

            timezone = st.selectbox('Select timezone?',tuple([s for s in pytz.all_timezones]))
            pred_time_utc = datetime.utcfromtimestamp(pred_time)

            converted_time = pred_time_utc.astimezone(tz.gettz(timezone)).strftime("%Y-%m-%d %H:%M:%S")
            st.markdown(f"<h3 style='text-align: center;'>Visible for {visible_duration} seconds from {converted_time}</h3>",unsafe_allow_html=True)
        else:
            st.write('Not a valid city')




from mpl_toolkits.basemap import Basemap # pip does not include this package, install by downloading the binary from web
import matplotlib.pyplot as plt
plt.rcParams['animation.ffmpeg_path'] = '/usr/bin/ffmpeg' # point to the extracted exe file of ffmpeg
import matplotlib.animation as animate
import streamlit as st
from issTrack.issTracking import TrackerISS
from mylogger.iss_logging import logger
import numpy as np
from geopy.geocoders import Nominatim

class BasemapPlot:

    def __init__(self, home_name,home_latitude, home_longitude):

        self.home_name = home_name
        self.home_latitude = home_latitude
        self.home_longitude = home_longitude

        # self.gps_location = gps_location

        figure, ax = plt.subplots(num="ISS Tracker",figsize=(14,8))
        self.the_plot = st.pyplot(plt,clear_figure=True)

    def create_plot(self):
        self.gps_location = TrackerISS.get_iss_lat_lon()

        try:
            self.m = Basemap(projection='ortho',
                lat_0=self.gps_location['latitude'],
                lon_0=self.gps_location['longitude'],
                resolution='c')
            self.m.fillcontinents(color='coral',lake_color='aqua')
            # draw parallels and meridians
            self.m.drawparallels(np.arange(-90.,91.,30.))
            self.m.drawmeridians(np.arange(-180.,181.,60.))
            self.m.drawcountries()
            self.m.drawmapboundary(fill_color='aqua')
            return self
        except Exception as e:
            print('Failure in basemap',e)

    def plot_location(self, speed_iss_pos):

        speed = speed_iss_pos[0]
        iss = speed_iss_pos[1]

        self.create_plot()
        x_pt, y_pt = self.m(self.gps_location['longitude'],self.gps_location['latitude'])
        self.point = self.m.plot(x_pt, y_pt,'bo')[0]
        self.point.set_data(x_pt,y_pt)
        plt.text(x_pt, y_pt, 'Lat:{} Lon:{}'.\
            format(\
                round(self.gps_location['latitude'],2),\
                round(self.gps_location['longitude'],2)))

        distance_to_home = TrackerISS.get_distance_btwn_locations(location_1 = (self.home_latitude, self.home_longitude),\
                                                                    location_2 = iss)
         
        location = Nominatim(user_agent="my-application",timeout=3)\
        .reverse('{},{}'.format(self.gps_location['latitude'],\
            self.gps_location['longitude']), language='en')
        try:
            country = location.raw['address']['country']
        except KeyError:
            country = 'the ocean'

        plt.title('ISS is currently above {} \n \
            Ground distance between {} and ISS is {}km.\n \
            Ground speed {} km/h' \
            .format(country,self.home_name,round(distance_to_home,2),round(speed,2) if speed > 1 else 'Calculating...'))
        self.the_plot.pyplot(plt,clear_figure=True)


import mysql.connector
from mysql.connector import errorcode
from mysql.connector import Error
from mylogger.iss_logging import logger

#mysql -P 3606 -h localhost -u root --password=root


class MySql:

    config = {
        'user': 'root',
        'password': 'root',
        'host': 'liveisstracker_db_1',
        'port': '3306',
        'database': 'dev'
        }
    def __init__(self):
        try:
            self.connection = mysql.connector.connect(**self.config)
            logger.info('DB connection successful')
        except mysql.connector.Error as err:
            if err.errno == errorcode.ER_ACCESS_DENIED_ERROR:
                logger.error('Incorrect user name or password')
            elif err.errno == errorcode.ER_BAD_DB_ERROR:
                logger.error('Database does not exist')
            elif err.errno == 2003:
                logger.error('Cant connect to Database')
            else:
                raise('Unknown Error connecting to Database')
    
    def insert_record(self,table_name,key_values):
        '''
        Function to insert records in the table.
        arg: table_name = Name of the table into which records to be inserted. (str)
        arg: key_values = Dictionary of key value pairs where key is the column name
                        value is the entry corresponding to that key.
        return: None'''

        if key_values == '':
            key_values = {}

        try:
            self.cursor = self.connection.cursor()
            add_record = ("INSERT INTO location "
                    "(lat,lon,datetime_id) "
                    " VALUES (%(lat)s, %(lon)s, %(datetime_id)s)")
            self.cursor.execute(add_record,key_values)
            self.connection.commit()
        except Exception as err:
            logger.error("Unable to insert records: Error: {}".format(err))
        
    def close_connection(self):
        self.cursor.close()
        self.connection.close()



"""
Author: Manoj Manivannan
Date 13-Jun-2020

This script plots the ground position of international space station on a spherical map.
The location of ISS is obtained from 'http://api.open-notify.org/iss-now.json'
The resulting Geo coordinates are plot using matplotlib.
"""


import time
import streamlit as st
from mylogger.iss_logging import logger
from issTrack.page_information import information
from issTrack.issTracking import *
from PlotMap.MapBasePlot import *
from PlotMap.PlotlyFig import get_plotly_figure, get_mapbox_token, display_pass_statistics

st.set_page_config(
    page_title="Live ISS Tracker", 
    page_icon='https://icons.iconarchive.com/icons/goodstuff-no-nonsense/free-space/512/international-space-station-icon.png', 
    layout='centered', 
    initial_sidebar_state='auto')


def main():
            
    try:
        st.title('International Space Station Tracker')
        st.markdown(information['header1'])
        st.markdown(information['what'])
        st.markdown(information['intro_source'],unsafe_allow_html=True)
        st.markdown(information['header2'])
        st.markdown(information['tech_spec'], unsafe_allow_html=True)
        st.markdown(information['intro_source'],unsafe_allow_html=True)

        iss = TrackerISS()
        live_show = st.radio("Show live tracking in orthographic",('Yes', 'No'), index=1)
        if live_show == 'Yes':
            home_name_st = st.text_input('Distance relative to (city)',value='')
            if home_name_st:
                home_name, home_lat, home_lon = get_city_location(home_name_st)
                
                earth = BasemapPlot(home_name,home_lat,home_lon)
        while live_show == 'Yes' and home_name_st:
            earth.plot_location(iss.get_speed_iss_pos())
            with st.spinner('Reloading in 5 seconds..'):
                time.sleep(5)

        mapbox_disable,mapbox_access_token = get_mapbox_token(env_var_name='MAPBOX_TOKEN')
        if not mapbox_disable:        
            st.write("Current ISS location in flat view")
            location = iss.get_speed_iss_pos()
            fig = get_plotly_figure(location,token=mapbox_access_token)

            if st.button('Refresh'):
                pass
            st.plotly_chart(fig)

        st.write("Predict next ISS pass through by city")
        predict_city = st.text_input('Enter city name',value='London')
        
        if predict_city:
            _, predict_city_lat, predict_city_lon = get_city_location(predict_city)

            pass_information=iss.get_pass_info_from_lat_lon(predict_city_lat,predict_city_lon)
            display_pass_statistics(pass_information)


    except Exception as e:
        logger.error('Failed {}'.format(e))
        raise Exception(e)


if __name__ == '__main__':
    main()


from unittest import TestCase, TextTestRunner, defaultTestLoader
from liveisstracker.issTrack.issTracking import TrackerISS
from urllib.error import URLError

###### MODENA #######################
home_name="MODENA"
home_lat=44.6501557
home_lon=10.8516923

class TestISSTracker(TestCase):

    @classmethod
    def setUpClass(self):
        print("Test ISS Tracker Start")
        self.tracker = TrackerISS()


    @classmethod
    def tearDownClass(self):
        print("Test ISS Tracker  Done")

    def test_default_assertTrue(self):
        # print("Default Test")
        self.assertTrue(True)

    def test_get_speed_iss_at_0_0(self):
        # print("Speed test speed=0")
        self.tracker.timestamp_pre = 0
        testing_mode = {'timestamp':1, 'latitude':0, 'longitude':0}
        self.assertEqual(self.tracker.get_speed_iss_pos(testing_mode = testing_mode)[0],0)

    def test_get_speed_iss_at_1_1(self):
        # print("Speed test iss at 1,1")
        self.tracker.timestamp_pre = 0
        self.tracker.lon_pre = 0
        self.tracker.lat_pre = 0
        testing_mode = {'timestamp':1, 'latitude':1, 'longitude':1}
        self.assertAlmostEqual(self.tracker.get_speed_iss_pos(testing_mode = testing_mode)[0],564838.4, places=1)

    def test_get_speed_iss_at_10_10(self):
        self.tracker.timestamp_pre = 0
        self.tracker.lon_pre = 0
        self.tracker.lat_pre = 0
        testing_mode = {'timestamp':1, 'latitude':10, 'longitude':10}
        self.assertAlmostEqual(self.tracker.get_speed_iss_pos(testing_mode = testing_mode)[0],5634392.7, places=0)

    def test_get_iss_location_invalid_link(self):
        # print("Fetch ISS location - Invalid link")
        testing_mode = {'iss_link':'http://api.open-notify.org/iss-no.json'}
        self.assertRaises(URLError, self.tracker.get_iss_lat_lon, testing_mode)

    def test_get_iss_distance_from_location_0_0_km(self):
        #
        testing_mode = {'location_1':(0,0), 'location_2':(0,0)}
        self.assertEqual(self.tracker.get_distance_btwn_locations(testing_mode=testing_mode),0)

    def test_get_iss_distance_from_location_1_1_km(self):
        #
        testing_mode = {'location_1':(0,0), 'location_2':(1,1)}
        self.assertAlmostEqual(self.tracker.get_distance_btwn_locations(testing_mode=testing_mode),156.89, places=1)

    def test_get_iss_distance_from_location_10_10_km(self):

        testing_mode = {'location_1':(0,0), 'location_2':(10,10)}
        self.assertAlmostEqual(self.tracker.get_distance_btwn_locations(testing_mode=testing_mode),1565.1, places=1)

    def test_get_pass_information_from_location_0_0(self):
        testing_mode={'timestamp':1, 'latitude':0, 'longitude':0}
        self.assertEqual(self.tracker.get_pass_info_from_lat_lon(testing_mode=testing_mode),'failure')

def run():
    TextTestRunner().run(defaultTestLoader.loadTestsFromTestCase(TestISSTracker))

if __name__ == '__main__':

    run()




import logging
#FORMAT = '%(asctime)s - %(levelname)s %(message)s'
#logging.basicConfig(format=FORMAT)
logger = logging.getLogger('LiveISStracker')
logger.propagate = False
logger.setLevel('INFO')
ch = logging.StreamHandler()
formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s')
ch.setFormatter(formatter)
logger.addHandler(ch)



"""
from tempfile import mkstemp
from shutil import move, copymode
from os import fdopen, remove

bad_words = ['basemap', 'Basemap','projection']

def replace(file_path, bad_words):
    #Create temp file
    fh, abs_path = mkstemp()
    with fdopen(fh,'w') as new_file:
        with open(file_path) as old_file:
            for line in old_file:
                if not any(bad_word in line for bad_word in bad_words):
                    new_file.write(line)
    #Copy the file permissions from the old file to the new file
    copymode(file_path, abs_path)
    #Remove original file
    remove(file_path)
    #Move new file
    move(abs_path, file_path)

print('Disable basemap from track_iss')
replace('liveisstracker/track_iss.py',bad_words)
"""



from setuptools import setup, find_packages

requirements = [l.strip() for l in open('requirements.txt').readlines()]
requirements.append('pytest')

setup(
    name='liveisstracker',
    url='https://gitlab.com/manojm18',
    version='0.0.2',
    author='Manoj Manivannan',
    author_email='manojm18@live.in',
    description='Sample Python Project build using Maven and CI/CD',
    packages=find_packages(),
    entry_points={
    'console_scripts': ['liveisstracker=liveisstracker.command_line:main'],
    },
    install_requires=requirements,
    include_package_data=True,
)





information = {
        'header1':'## What is it ?',
        'what':'The International Space Station (ISS) is a modular space station (habitable artificial satellite) in low Earth orbit. It is a multinational collaborative project involving five participating space agencies: NASA (United States), Roscosmos (Russia), JAXA (Japan), ESA (Europe), and CSA (Canada). The ownership and use of the space station is established by intergovernmental treaties and agreements. The station serves as a microgravity and space environment research laboratory in which scientific research is conducted in astrobiology, astronomy, meteorology, physics, and other fields. The ISS is suited for testing the spacecraft systems and equipment required for possible future long-duration missions to the Moon and Mars.',
        'intro_source':"<h6 style='text-align: right;'>Source: <a href=\"https://en.wikipedia.org/wiki/International_Space_Station\">Wikipedia</a></h6>",
        'header2':'## Some facts !',
        'tech_spec':'''<style>\
                        table, th, td {\
                            border: 1px solid black;\
                            border-collapse: collapse;\
                            background-color: #f1f1f1;\
                            }\
                        th, td {\
                            padding: 15px;\
                            text-align: left;\
                            }\
                        </style>\
                        <table>\
                            <tr><th>Specification</th>              <th>Value</th></tr>\
                            <tr><td>Launch</td>						<td>20 November 1998; 22 years ago</td></tr>\
                            <tr><td>Launch pad</td>				    <td>Baikonur, Site 1/5 and Site 81/23, Kennedy, LC-39 and CCAFS SLC-41 (future)</td></tr>\
                            <tr><td>Mass</td>						<td>419,725 kg</td></tr>\
                            <tr><td>Length</td>						<td>73 m</td></tr>\
                            <tr><td>Width</td>						<td>109 m</td></tr>\
                            <tr><td>Pressurised volume</td>			<td>915.6 m<sup>3</sup></td></tr>\
                            <tr><td>Atmospheric pressure</td>		<td>101.3 kPa or 1.0 atm [oxygen 21%, nitrogen 79%]</td></tr>\
                            <tr><td>Perigee altitude</td>			<td>418 km</td></tr>\
                            <tr><td>Apogee altitude</td>			<td>422 km</td></tr>\
                            <tr><td>Orbital inclination</td>		<td>51.63°</td></tr>\
                            <tr><td>Orbital speed</td>				<td>~ 7.66 km/s or 27,600 km/h</td></tr>\
                            <tr><td>Orbital period</td>				<td>92.68 minutes</td></tr>\
                            <tr><td>Orbits per day</td>				<td>15.54</td></tr>\
                            <tr><td>Days in orbit</td>				<td>22 years, 2 months, 30 days</td></tr>\
                            <tr><td>Days occupied</td>				<td>20 years, 3 months, 17 days</td></tr>\
                            <tr><td>No. of orbits</td>				<td>125,964 as of December 2020</td></tr>\
                            <tr><td>Orbital decay</td>				<td>2 km/month</td></tr>\
                            </table>'''
}
                            